/*
 * Sven Kratz, Michael Rohs (c) 2011
 */
package de.lmu.mobilehci.gesturewizards;

import java.util.Random;
import java.util.concurrent.ConcurrentLinkedQueue;

import android.content.Context;
import android.os.Handler;
import android.os.Message;
import android.os.Vibrator;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.FrameLayout;
import android.widget.TextView;

/**
 * Represents a wizard. This is a singleton class, i.e. the class itself creates
 * two instances. It has a private constructor, which prevents that more
 * instances can be created. This is a common design pattern. The wizard can
 * cast spells, attack another wizard, store points, etc. This class also
 * implements the functionality of the automatic wizard. If its strategy is
 * extended, it might be beneficial to put this logic into its own class (or
 * derive a class SamrtWizard from this class).
 */
public class Wizard {

	private GestureWizardsActivity context;
	
	public int points = 10;
	private Spell defenseSpell = null;
	
	public static Wizard whiteWizard = new Wizard();
	public static Wizard blackWizard = new Wizard();

	private Vibrator vibrator = null;
	
	private View view = null;

	// only the white and black wizard should be created, no others
	private Wizard() {}

	public void setContext(GestureWizardsActivity c) {
		context = c;
		vibrator = (Vibrator) c.getSystemService(Context.VIBRATOR_SERVICE);

		if (this == Wizard.whiteWizard) {
			view = context.findViewById(R.id.wizard1);
		} else {
			view = context.findViewById(R.id.wizard2);
		}
	
	}
	
	public Spell castSpell(SpellInfo si, int animationResource) {
		FrameLayout spellsLayout = null;
		Spell spell = null;
		
		cancelDefenseSpell();
		
		if (this == Wizard.whiteWizard) {
			spellsLayout = (FrameLayout) context.findViewById(R.id.whiteWizardSpells);
			spell = new Spell(context, si, this, animationResource);			
			Animation animation = AnimationUtils.loadAnimation(context, R.anim.wizardwhitecastspellanima);
			view.startAnimation(animation);
		} else {
			spellsLayout = (FrameLayout) context.findViewById(R.id.blackWizardSpells);
			spell = new Spell(context, si, this, animationResource);			
			Animation animation = AnimationUtils.loadAnimation(context, R.anim.wizardblackcastspellanima);
			view.startAnimation(animation);
		}

		if (spell.isDefenseSpell()) {
			defenseSpell = spell;
		}

		SpellView spellv = spell.getView();
		spellsLayout.addView(spellv);
		spellv.animate();
		
		return spell;
	}    

	private void cancelDefenseSpell() {
		if (defenseSpell != null) {
			defenseSpell.cancelAnimation();
			defenseSpell = null;
		}
	}
	
	public void spellDidFinish(Spell s) {
		if (s.isDefenseSpell()) {
			defenseSpell = null;
		} else { // is attack spell
			Wizard opponent = (this == Wizard.whiteWizard) ? Wizard.blackWizard : Wizard.whiteWizard; 
			boolean success = opponent.attack(s);
			// play animation on success
			// ...
			
			// update points text view
			TextView tv = (TextView) context.findViewById(R.id.gameInfo);
			String output = Integer.toString(whiteWizard.points) + " : " + Integer.toString(blackWizard.points);
			tv.setText(output);
		}
	}
	
	public boolean attack(Spell attackSpell) {
		if (context.gameOver) return false;
		if (!attackSpell.isDefenseSpell()) {
			if (defenseSpell == null || !defenseSpell.matches(attackSpell)) {
				points--;
				if (points > 0) {
					if (this == whiteWizard) {
						Animation whiteWizardHitAnimation = AnimationUtils
								.loadAnimation(context,
										R.anim.wizardwhitehitanima);
						View wizardView1 = context.findViewById(R.id.wizard1);
						wizardView1.startAnimation(whiteWizardHitAnimation);
					} else {
						Animation blackWizardHitAnimation = AnimationUtils
								.loadAnimation(context,
										R.anim.wizardblackhitanima);
						View wizardView2 = context.findViewById(R.id.wizard2);
						wizardView2.startAnimation(blackWizardHitAnimation);
					}
				} else { // 0 points, game over
					context.gameOver = true;
					stopAutomaticWizard();
					if (this == whiteWizard) {
						Animation whiteWizardHitAnimation = AnimationUtils
								.loadAnimation(context,
										R.anim.wizardwhitelostanima);
						View wizardView1 = context.findViewById(R.id.wizard1);
						wizardView1.startAnimation(whiteWizardHitAnimation);
					} else {
						Animation blackWizardHitAnimation = AnimationUtils
								.loadAnimation(context,
										R.anim.wizardblacklostanima);
						View wizardView2 = context.findViewById(R.id.wizard2);
						wizardView2.startAnimation(blackWizardHitAnimation);
					}
				}
				vibrator.vibrate(100);
				return true;
			}
		}
		return false;
	}
	
	private Handler tickHandler = null;
	private ConcurrentLinkedQueue<Spell> opponentSpells = new ConcurrentLinkedQueue<Spell>();
	
	public void startAutomaticWizard() {

		tickHandler = new Handler() {
			public void handleMessage(Message msg) {
				
				if (!opponentSpells.isEmpty()) {
					// find matching defensive spell
					// cast matching one...
				}
				opponentSpells.clear();
				
				int n = context.spellInfoMap.size();
				Random rnd = new Random();
				int i = rnd.nextInt(n);
				SpellInfo si = (SpellInfo) context.spellInfoMap.values().toArray()[i];
				if (si.defensive) {
					castSpell(si, R.anim.blackdefspellanima);
				} else {
					castSpell(si, R.anim.blackspellanima);
				}
				
				sendMessageDelayed(obtainMessage(1), 4000);
			}
    	};
    	
        tickHandler.sendMessageDelayed(tickHandler.obtainMessage(1), 4000);
    		
	}
	
	public void stopAutomaticWizard() {
		if (tickHandler != null) {
			tickHandler.removeMessages(1);
		}
	}
	
	public void opponentDidLaunchAttackSpell(Spell s) {
		// react to incoming spell...
		// add s to queue, handle queue in handleMessage
		if (!s.isDefenseSpell()) {
			opponentSpells.add(s);
		}
	}
	
	public void reset() {
		points = 10;
		view.clearAnimation();
	}
	
}
